package de.unibremen.ggj2014;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.physics.box2d.Body;
import com.badlogic.gdx.physics.box2d.BodyDef.BodyType;
import com.badlogic.gdx.physics.box2d.Filter;


public class Player extends Entity {
    public static final float COUNT_DOWN = 8f;
    public static final float RESPAWN = 1f;
    public static final int NUM_PARTICLES = 5;

	private float touchingGround;
	public boolean isLookingRight = true;
	
	private float remainingSecs;
	private float respawnSecs;
	
	private char blockedDirection;
	
    private final static String[] ANIMATIONS = new String[] {
        "data/PlayerIdle.png",
        "player_walk_left",
        "player_walk_right",
        "player_jump"
    };
    
    private final Animation countDown;
    
    Animation idleRight;
    Animation idleLeft;
    Animation runLeft;
    Animation runRight;
    Animation jumpLeft;
    Animation jumpRight;
    
    private float timeInLevel;
    private static float timeInGame;
    
    private int lifesInLevel;
    private static int lifesInGame;

	public Player(float x, float y) {
	    super((Animation) null, new Physics(x, y, 0.49f, true, 0f));
		touchingGround = 0;
		
		countDown = new Animation("data/countDown.png", TILE_SIZE_IN_PX / 2, 1.0f);
		countDown.disableLoop();
		
		this.idleRight = new Animation("data/PlayerIdle.png", TILE_SIZE_IN_PX);
		this.idleLeft = new Animation("data/PlayerIdleLeft.png", TILE_SIZE_IN_PX);
		this.runLeft = new Animation("data/PlayerRunLeft.png", TILE_SIZE_IN_PX, 0.075f);
		this.runRight = new Animation("data/PlayerRunRight.png", TILE_SIZE_IN_PX, 0.075f);
		this.jumpLeft = new Animation("data/PlayerJumpLeft.png", TILE_SIZE_IN_PX, 0.15f);
		this.jumpRight = new Animation("data/PlayerJumpRight.png", TILE_SIZE_IN_PX, 0.15f);
		
	    setAnimation(idleRight);
	}
	
	@Override
    public void init() {
        super.init();

        getBody().getFixtureList().get(0).setFriction(5);
        blockedDirection = '0';
	}
	
	@Override
	public void draw(Batch batch) {
	    if (!isDead()) {
	        super.draw(batch);
            countDown.draw(batch, getX() * Entity.TILE_SIZE_IN_PX, (getY() + 1) * Entity.TILE_SIZE_IN_PX);
	    }
	}
	
	private static Filter filter;
	@Override
	public Filter getFilter() {
		if(filter == null){
	    	filter = new Filter();
	    	filter.categoryBits = Physics.PLAYER; // I am a
	    	filter.maskBits = Physics.GROUND | Physics.HAZARD | Physics.TRIGGER; // I can collide with
	    }
		return filter;
	}
	
	public boolean isTouchingGround() {
		return touchingGround == 0;
	}

	public void setTouchingGround(boolean touchingGround) {
		if(this.touchingGround != 0 && touchingGround){
			this.touchingGround = 1f/59f;
		}
		if(!touchingGround){
			this.touchingGround = -1;
		}
	}
	
	@Override
	public void update(float deltaTime) {
		super.update(deltaTime);
		this.timeInGame += deltaTime;
		this.timeInLevel += deltaTime;
		if (!isDead()) {
		    this.remainingSecs -= deltaTime;
		    if(remainingSecs <= 0) {
			this.die(null, getBody().getLinearVelocity());
		    }
		}
		
		if(isDead()) {
		    respawnSecs -= deltaTime;
		    if(respawnSecs <= 0) {
			this.respawn();
		    }
		}
		
		if(touchingGround > 0){
			if(deltaTime > touchingGround)
				touchingGround = 0;
			else
				touchingGround -= deltaTime;
		}
		
		if(Gdx.input.isKeyPressed(Input.Keys.R)){
		    respawn();
		}
		
		countDown.update(deltaTime);
		
		if (!this.isTouchingGround()) {
			if (isLookingRight)
				this.setAnimation(jumpRight);
			else
				this.setAnimation(jumpLeft);
		} else if (this.getBody().getLinearVelocity().len2() < 1 && this.isTouchingGround()) {
			if (!isLookingRight)
				this.setAnimation(idleLeft);
			else
				this.setAnimation(idleRight);
		} else {
			if (isLookingRight)
				this.setAnimation(runRight);
			else
				this.setAnimation(runLeft);
		}
	}
	
	public static Player spawn() {
	    float x = GGJ2014Game.singleton.getLevel().getCurrentSpawnPosition().getX();
	    float y = GGJ2014Game.singleton.getLevel().getCurrentSpawnPosition().getY();
	    Player player = new Player(x, y);
	    player.addBehaviour(new UserInputBehaviour(player));
	    GGJ2014Game.singleton.cameraCenter = player;
	    player.remainingSecs = COUNT_DOWN;
	    player.timeInLevel = 0;
	    player.lifesInLevel = 0;

	    return player;
	}
	
	public void die(final Entity killer) {
	    die(killer, null);
	}
	
	public void die(final Entity killer, Vector2 v) {
	    this.lifesInLevel++;
	    this.lifesInGame++;
        float minS = 0.7f;
        float maxS = 2.0f;
        float minR = -20.0f;
        float maxR = 20.0f;
        
	    if (v == null || v.len2() < 1) {
	        v = new Vector2(0, 3.5f);
	        minR = -60.0f;
	        maxR = 60.0f;
	    }
	    
	    for (int i = 0; i < NUM_PARTICLES; ++i) {
	        final Vector2 vp = v.cpy().scl(GGJ2014Game.rand(minS, maxS)).rotate(GGJ2014Game.rand(minR, maxR));
	        GGJ2014Game.singleton.particles.getParticle(getX(), getY(), vp);
	    }
	    
	    float bsY = getY() + .5f;
	    if (isTouchingGround()) {
	        bsY = getY();
	    }
	    
	    GGJ2014Game.singleton.particles.getBloodSplash(BloodSplash.Direction.LEFT, getX() - .5f, bsY);
	    GGJ2014Game.singleton.particles.getBloodSplash(BloodSplash.Direction.RIGHT, getX() + .5f, bsY);
        
	    final Body b = getBody();
	    GGJ2014Game.singleton.addPhysicsUpdate(new Runnable() {
            @Override
            public void run() {
                b.setLinearVelocity(0, 0);
                b.getFixtureList().get(0).setSensor(true);
                b.setType(BodyType.StaticBody);
                if(killer!=null) {
                    GGJ2014Game.singleton.cameraCenter = killer;
                }
                setAnimation(null);
                
                
            }
        });
	    GGJ2014Game.singleton.soundManager.die.play();
	    this.respawnSecs = RESPAWN;
	}
	
	public void respawn() {
	    final float x = GGJ2014Game.singleton.getLevel().getCurrentSpawnPosition().getX();
        final float y = GGJ2014Game.singleton.getLevel().getCurrentSpawnPosition().getY();
        final Player p = this;
        countDown.reset();
        GGJ2014Game.singleton.addPhysicsUpdate(new Runnable() {
            @Override
            public void run() {
                p.getBody().setTransform(x, y, 0);
                p.getBody().getFixtureList().get(0).setSensor(false);
                p.getBody().setType(BodyType.DynamicBody);
                setAnimation(idleRight);

                GGJ2014Game.singleton.cameraCenter = p;
                p.remainingSecs = COUNT_DOWN;
            }
        });
	}
	
	public boolean isDead(){
		return getBody().getFixtureList().get(0).isSensor();
	}
	
	public boolean leftBlocked(){
		return blockedDirection == 'l';
	}
	
	public boolean rightBlocked(){
		return blockedDirection == 'r';
	}
	
	public void setBlockedDirection(char direction){
		blockedDirection = direction;
    }
  
	public int getLifesInLevel() {
	    return this.lifesInLevel;
	}
	
	public int getLifesInGame() {
	    return this.lifesInGame;
	}
	
	public float getTimeInLevel() {
	    return this.timeInLevel;
	}
	
	public float getTimeInGame() {
	    return this.timeInGame;
	}
}
